/*
** Copyright (C) 2002 Erik de Castro Lopo <erikd@mega-nerd.com>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include	<stdio.h>
#include 	<unistd.h>
#include 	<fcntl.h>
#include 	<sys/ioctl.h>
#include 	<sys/soundcard.h>
#include 	<math.h>
		
int	
open_dev_dsp (int channels, int srate) 
{	int fd, stereo, temp, error ;

	if ((fd = open ("/dev/dsp", O_WRONLY, 0)) == -1) 
	{	perror ("open_dev_dsp 1 ") ;  
		exit (1) ;  
		} ;

	stereo = 0 ;
	if (ioctl (fd, SNDCTL_DSP_STEREO, &stereo) == -1)
	{ 	/* Fatal error */
		perror ("open_dev_dsp 2 ") ;
		exit (1);
		} ;

	if (ioctl (fd, SNDCTL_DSP_RESET, 0))
	{	perror ("open_dev_dsp 3 ") ;
		exit (1) ;
		} ;
	
	temp = 16 ; 
	if ((error = ioctl (fd, SOUND_PCM_WRITE_BITS, &temp)) != 0)
	{	perror ("open_dev_dsp 4 ") ;
		exit (1) ;
		} ;

	if ((error = ioctl (fd, SOUND_PCM_WRITE_CHANNELS, &channels)) != 0)
	{	perror ("open_dev_dsp 5 ") ;
		exit (1) ;
		} ;

	if ((error = ioctl (fd, SOUND_PCM_WRITE_RATE, &srate)) != 0)
	{	perror ("open_dev_dsp 6 ") ;
		exit (1) ;
		} ;

	if ((error = ioctl (fd, SNDCTL_DSP_SYNC, 0)) != 0)
	{	perror ("open_dev_dsp 7 ") ;
		exit (1) ;
		} ;

	return 	fd ;
} /* open_dev_dsp */

int	
open_dev_mixer (void) 
{	int fd ;

	if ((fd = open ("/dev/mixer", O_RDWR, 0)) == -1) 
	{	perror ("open_dev_mixer 1 ") ;  
		exit (1) ;  
		} ;

	return 	fd ;
} /* open_dev_mixer */

void
set_mixer (int mixer, float master, float pcm) 
{	static float last_master = 0.0, last_pcm = 0.0 ;

	int temp ;
	
	/*	The ALSA sound drivers are very slow to respond to the OSS mixer 
	**	ioctls. Therefore only call the ioctl if the value actually needs 
	**	to be updated.
	*/
	
	if (fabs (last_master - master) > 0.9)
	{	temp = master ;
		if (temp < 0)
			temp = 0 ;
		if (temp > 100)
			temp = 100 ;

		/* Fiddle the values so bost left and right channels have the same value. */
		temp = temp | (temp << 8) ;
		ioctl (mixer, MIXER_WRITE (SOUND_MIXER_VOLUME), &temp) ;
		
		last_master = master ;
		} ;

	if (fabs (last_pcm - pcm) > 0.9)
	{	temp = pcm ;
		if (temp < 0)
			temp = 0 ;
		if (temp > 100)
			temp = 100 ;

		/* Fiddle the values so bost left and right channels have the same value. */
		temp = temp | (temp << 8) ;
		ioctl (mixer, MIXER_WRITE (SOUND_MIXER_PCM), &temp) ;
		
		last_pcm = pcm ;
		} ;

	return ;
} /* set_mixer */

void
get_mixer (int mixer, float *master, float *pcm) 
{	int temp ;

	ioctl (mixer, MIXER_READ (SOUND_MIXER_VOLUME), &temp) ;
	temp = temp & 0xFF ;
	*master = temp ;

	ioctl (mixer, MIXER_READ (SOUND_MIXER_PCM), &temp) ;
	temp = temp & 0xFF ;
	*pcm = temp ;

	return ;
} /* set_mixer */
