/*
** Copyright (C) 2002 Erik de Castro Lopo <erikd@mega-nerd.com>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include	<stdio.h>
#include	<math.h>

#include	<gtk/gtk.h>

#include	"gui.h"

#define	ADJUST_SIZE		300
#define Y_SPACING		20

static GtkWidget *window = NULL ;

static void	make_hscale (GUI_ELEMENT *element, GtkObject *adjust, GtkWidget *box) ;

static void handle_hscale (GtkAdjustment *adj, GUI_ELEMENT *element) ;
static void handle_flipflop (GtkWidget *button, GUI_ELEMENT *element) ;

int
create_gui (char *title, GUI_ELEMENT *elements, int count)
{	GtkWidget *label, *button, *box1, *box2 ;
	GtkObject *adjust ;

	const char	*str ;
	int 		k ;

	if (window)
		return 0 ;

	gtk_init (NULL, NULL) ;

	/* Create main window and connect window destroy handler. */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL) ;
	gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (gtk_main_quit), NULL) ;
	gtk_window_set_title (GTK_WINDOW (window), title) ;

	box1 = gtk_vbox_new (FALSE, 0) ;
	gtk_container_add (GTK_CONTAINER (window), box1) ;
	gtk_widget_show (box1) ;

	for (k = 0 ; k < count ; k++)
	{	if (elements [k].type == HORIZ_SEPARATOR)
		{   GtkWidget *separator;

			separator = gtk_hseparator_new () ;
    		gtk_box_pack_start (GTK_BOX (box1), separator, TRUE, TRUE, 0) ;

			gtk_widget_set_usize (GTK_WIDGET (separator), ADJUST_SIZE, 15) ;

    		gtk_widget_show (separator) ;

			continue ;
			} ;

		box2 = gtk_hbox_new (FALSE, 10) ;
		gtk_container_set_border_width (GTK_CONTAINER (box2), 10) ;
		gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0) ;
		gtk_widget_show (box2) ;

		label = gtk_label_new (elements [k].name) ;
		gtk_widget_set_usize (GTK_WIDGET (label), 180, Y_SPACING) ;
		gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0) ;
		gtk_widget_show (label) ;

		switch (elements[k].type)
		{	case LOG_ADJUSTMENT :
					if (elements [k].adj.lower < 0.01)
						elements [k].adj.lower = 0.01 ;
					adjust = gtk_adjustment_new (log10 (*(elements [k].adj.mod_value)), 
						log10 (elements [k].adj.lower), log10 (elements [k].adj.upper), 
						log10 (elements [k].adj.step), 0.0, 0.0
						) ;
					make_hscale (&(elements [k]), adjust, box2) ;
					break ;

			case LINEAR_ADJUSTMENT :
					adjust = gtk_adjustment_new (*(elements [k].adj.mod_value), 
						elements [k].adj.lower, elements [k].adj.upper, 
						elements [k].adj.step, 0.0, 0.0
						) ;
					make_hscale (&(elements [k]), adjust, box2) ;
					break ;

			case FLIPFLOP_CHOICE :
					str = "Label" ;
					if (*(elements[k].ff.mod_value) < MAX_FLIPFLOP_NAMES &&
						elements [k].ff.names [*(elements[k].ff.mod_value)])
						str = elements [k].ff.names [*(elements[k].ff.mod_value)] ;

					elements [k].ff.label = gtk_label_new (str) ;
					gtk_widget_show (elements [k].ff.label) ;

					button = gtk_button_new () ;
					
					gtk_container_add (GTK_CONTAINER (button), elements [k].ff.label);

					gtk_widget_set_usize (GTK_WIDGET (button), ADJUST_SIZE, Y_SPACING) ;

					gtk_signal_connect (GTK_OBJECT (button), "clicked",
						GTK_SIGNAL_FUNC (handle_flipflop), &(elements [k]));

					gtk_box_pack_start (GTK_BOX (box2), button, FALSE, FALSE, 0) ;
					gtk_widget_show (button) ;
					break ;

			default : break ;	
			} ; /* switch */
		} ; /* for */

	gtk_widget_show (window) ;

	gtk_main () ;

	return 0 ;
} /* create_gui */

static void
make_hscale (GUI_ELEMENT *element, GtkObject *adjust, GtkWidget *box)
{	char 		buffer [32] ;
	GtkWidget	*hscale ;
	GtkWidget 	*label ;

	hscale = gtk_hscale_new (GTK_ADJUSTMENT (adjust)) ;
	gtk_widget_set_usize (GTK_WIDGET (hscale), ADJUST_SIZE, Y_SPACING) ;

	gtk_scale_set_draw_value (GTK_SCALE (hscale), FALSE) ;
	gtk_signal_connect (GTK_OBJECT (adjust), "value_changed", 
		GTK_SIGNAL_FUNC (handle_hscale), element
		);

	gtk_box_pack_start (GTK_BOX (box), hscale, TRUE, TRUE, 0) ;
	gtk_widget_show (hscale) ;

	snprintf (buffer, sizeof (buffer), "%.1f", *(element->adj.mod_value)) ;
	label = gtk_label_new (buffer) ;
	gtk_widget_set_usize (GTK_WIDGET (label), 60, Y_SPACING) ;
	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0) ;
	gtk_widget_show (label) ;

	element->adj.label = label ;

	handle_hscale (GTK_ADJUSTMENT (adjust), element) ;
} /* make_hscale */

/*==============================================================================
*/

static void
handle_hscale (GtkAdjustment *adj, GUI_ELEMENT *element)
{	char 	buffer [64] ;
	gfloat 	value ;

	value = GTK_ADJUSTMENT (adj)->value ;

	switch (element->type)
	{	case LOG_ADJUSTMENT :
				value = pow (10.0, value) ;

				if (element->adj.mod_value)
					*(element->adj.mod_value) = value ;

				snprintf (buffer, sizeof (buffer), "%.1f", value) ;
				break ;

		case LINEAR_ADJUSTMENT :
				if (element->adj.mod_value)
					*(element->adj.mod_value) = value ;

				snprintf (buffer, sizeof (buffer), "%.1f", GTK_ADJUSTMENT (adj)->value) ;
				break ;
		} ;

	gtk_label_set_text (GTK_LABEL (element->adj.label), buffer) ;
} /* handle_hscale */

static void 
handle_flipflop (GtkWidget *button, GUI_ELEMENT *element)
{	const char *str ;

	*(element->ff.mod_value) = (*(element->ff.mod_value) + 1) % MAX_FLIPFLOP_NAMES ;
	if (! element->ff.names [*(element->ff.mod_value)])
		*(element->ff.mod_value) = 0 ;

	str = "dummy" ;
	if (element->ff.names [*(element->ff.mod_value)])
		str = element->ff.names [*(element->ff.mod_value)] ;

	gtk_label_set_text (GTK_LABEL (element->ff.label), str) ;
} /* handle_flipflop */
