/*
** Copyright (C) 2002 Erik de Castro Lopo <erikd@mega-nerd.com>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation; either version 2.1 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include 	<pthread.h>
#include	<stdio.h>
#include	<unistd.h>
#include	<math.h>

#include	<sndfile.h>
#include	<complex.h>

#include	"gui.h"
#include	"common.h"
#include	"audio_out.h"

#define		BUFFER_SIZE		512

static void *gui_thread_proc (void*) ;
static void process_audio (SNDFILE *sndfile, int devdsp, int samplerate) ;

static char title [] = "Tremelo (Amplitude Modulation)" ;

static int	end_processing = 0 ;

static float	master_vol = 80.0, pcm_vol = 80.0 ;
static float	frequency = 5.0, tremelo_depth = 10.0 ;

static GUI_ELEMENT elements [] = 
{	{	LINEAR_ADJUSTMENT,	"Master volume (%)"	, 
		{	0.0, 100.0, 0.1, NULL, &master_vol	},	
		},
	{	LINEAR_ADJUSTMENT,	"PCM volume (%)"	, 
		{	0.0, 100.0, 0.1, NULL, &pcm_vol },	
		},
	{	HORIZ_SEPARATOR,	},
	{	LOG_ADJUSTMENT,	"Frequency (Hz)"		, 
		{	20.0, 20000.0, 0.1, NULL, &frequency },	
		},
	{	LOG_ADJUSTMENT,	"Depth (%)"			, 
		{	0.0, 100.0, 0.1, NULL, &tremelo_depth },	
		},
} ;

int 
main (int argc, char *argv[])
{	pthread_t	gui_thread ;
	int			devdsp = 0 ;
	
	if (pthread_create (&gui_thread, NULL, gui_thread_proc, NULL))
	{	printf ("Error creating screen thread.\n") ;	
		exit (1) ;
		} ;
	
	sleep (1) ; /* Pause for the GUI to start. */
	
	process_audio (NULL, devdsp, 44100) ;

	return 0 ;
} /* main */

static void *
gui_thread_proc (void* param)
{	
	/* This does not return until its time to kill program. */
	create_gui (title, elements, sizeof (elements) / sizeof (GUI_ELEMENT)) ;

	/* Signal all other threads that its time to die. */
	end_processing = 1 ;
	
	pthread_exit (NULL) ;
	
	return NULL ;
} /* gui_thread_proc */

static void 
process_audio (SNDFILE *sndfile, int devdsp, int samplerate)
{	while (! end_processing)
	{	sleep (1) ;
		} ;

} /* process_audio */

