function signal = sinegen (samples, fs, freq, phase, mag);
# sinegen    Generates a signal consisting of a sum of sine waves with the 
#            specified frequencies and magnitudes.
#
#              	signal = sinegen (samples, fs, freq, phase, mag)
#              	signal = sinegen (samples, fs, freq, phase)
#               signal = sinegen (samples, fs, freq) ; 		% All magnitues == 1
#            
#              samples - number of samples to generate
#              fs      - sampling rate (Hz)
#              freq    - vector containing the required frequencies (Hz)
#              phase   - vector containing the required phases in rad/sec
#              mag     - vector containing the required magitudes

#      Erik de Castro Lopo    9-1-1994

if nargin < 3,
    error ('Insufficient input arguments.') 
    endif

if nargin < 4,
	phase = zeros (size(freq)) ;
    mag = ones (size (freq)) ;
    endif

if nargin < 5,
    mag = ones (size (freq)) ;
    endif
    
t = 2 * pi / fs * (0:samples-1) ;

signal = zeros (size (t)) ;                # make output vector of the correct size

freq = freq (1:length(mag)) ;

for k = 1:length (freq),
    signal = signal + mag(k) * sin (freq(k) * t + phase (k)) ;
    endfor

endfunction
