/*
** Copyright (C) 2002 Erik de Castro Lopo <erikd@zip.com.au>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include	<stdio.h>
#include 	<unistd.h>

#include 	<fcntl.h>
#include 	<sys/ioctl.h>
#include 	<sys/soundcard.h>

#include	<sndfile.h>

#define	BUFFER_LEN		1024

/*------------------------------------------------------------------------------
**	Linux/OSS functions for playing a sound.
*/

static	int	open_dsp_device (int channels, int srate) ;

/*==============================================================================
**	Main function.
*/

int 
main (int argc, char *argv [])
{	static short buffer [BUFFER_LEN] ;
	SNDFILE *sndfile ;
	SF_INFO sfinfo ;
	int		bit_width, audio_device, readcount ;

	if (argc != 3)
	{	printf ("Usage : %s -R:B:C <input sound file>\n\n", argv [0]) ;
		printf ("	where R : sample rate\n"
				"	      B : bit width (16, 24 or 32)\n"
				"	      C : channels\n\n"
				) ;
		return 1 ;
		} ;
		
	if (sscanf (argv [1], "-%d:%d:%d", &(sfinfo.samplerate), &bit_width, &(sfinfo.channels)) != 3)
	{	printf ("Error : Could not parse '%s'\n", argv [1]) ; 
		exit (1) ;
		} ;
	
	/* Processor native endian-ness. */
	switch (bit_width)
	{	case	16 :
				sfinfo.format = SF_FORMAT_RAW | SF_FORMAT_PCM_16 ;
				break ;
				
		case	24 :
				sfinfo.format = SF_FORMAT_RAW | SF_FORMAT_PCM_24 ;
				break ;
				
		case	32 :
				sfinfo.format = SF_FORMAT_RAW | SF_FORMAT_PCM_32 ;
				break ;
				
		default :
				printf ("Error : bad bit width (%d)\n", bit_width) ;
				exit (1) ;
		} ;
	
	if (! (sndfile = sf_open_read (argv [2], &sfinfo)))
	{	sf_perror (NULL) ;
		exit (1) ;
		} ;
			
	if (sfinfo.channels < 1 || sfinfo.channels > 2)
	{	printf ("Error : channels = %d.\n", sfinfo.channels) ;
		exit (1) ;
		} ;		
		
	audio_device = open_dsp_device (sfinfo.channels, sfinfo.samplerate) ;
		
	while ((readcount = sf_read_short (sndfile, buffer, BUFFER_LEN)))
			write (audio_device, buffer, readcount * sizeof (short)) ;
			
	sf_close (sndfile) ;
	close (audio_device) ;

	return 0 ;
} /* main */
		
static int	
open_dsp_device (int channels, int srate) 
{	int fd, stereo, temp, error ;

	if ((fd = open ("/dev/dsp", O_WRONLY, 0)) == -1) 
	{	perror("open_dsp_device 1 ") ;  
		exit (1) ;  
		} ;

	stereo = 0 ;
	if (ioctl (fd, SNDCTL_DSP_STEREO, &stereo) == -1)
	{ 	/* Fatal error */
		perror("open_dsp_device 2 ") ;
		exit (1);
		} ;

	if (ioctl (fd, SNDCTL_DSP_RESET, 0))
	{	perror ("open_dsp_device 3 ") ;
		exit (1) ;
		} ;
	
	temp = 16 ; 
	if ((error = ioctl (fd, SOUND_PCM_WRITE_BITS, &temp)) != 0)
	{	perror ("open_dsp_device 4 ") ;
		exit (1) ;
		} ;

	if ((error = ioctl (fd, SOUND_PCM_WRITE_CHANNELS, &channels)) != 0)
	{	perror ("open_dsp_device 5 ") ;
		exit (1) ;
		} ;

	if ((error = ioctl (fd, SOUND_PCM_WRITE_RATE, &srate)) != 0)
	{	perror ("open_dsp_device 6 ") ;
		exit (1) ;
		} ;

	if ((error = ioctl (fd, SNDCTL_DSP_SYNC, 0)) != 0)
	{	perror ("open_dsp_device 7 ") ;
		exit (1) ;
		} ;

	return 	fd ;
} /* open_dsp_device */
 

